@@ -23,6 +23,7 @@
  * \param scene Graphics scene to export (e.g. RGraphicsSceneQt)
  * \param fileName File name for exported bitmap. Extension determines format.
  * \param properties Various properties:
+ *
  *  properties["width"]: width of bitmap in pixels (ignored if resolution is present)
  *  properties["height"]: height of bitmap in pixels (ignored if resolution is present)
  *  properties["resolution"]: resolution in pixels / drawing unit
@@ -33,7 +34,10 @@
  *  properties["quality"]: Export quality (0..100), JPEG only
  *  properties["monochrome"]: true: Export as black / white
  *  properties["grayscale"]: true: Export as grayscale
- *  properties["window"]: RBox: window to export in drawing coorirdinates
+ *  properties["window"]: RBox: window to export in drawing coordinates
+ *  properties["entityids"]: Array: zoom to bounding box of given entities
+ *  properties["initView"]: Callback to initialize view
+ *
  * \param view Optional graphics view to use.
  */
 function exportBitmap(doc, scene, fileName, properties, view) {
@@ -74,8 +78,18 @@ function exportBitmap(doc, scene, fileName, properties, view) {
         properties["noweightmargin"] = false;
     }
 
+    if (typeof(properties["zoomall"])==="undefined") {
+        properties["zoomall"] = false;
+    }
+
     if (properties["resolution"]) {
-        var bb = doc.getBoundingBox(true, true);
+        var bb;
+        if (typeof(properties["entityids"])!=="undefined") {
+            bb = doc.getEntitiesBox(properties["entityids"]);
+        }
+        else {
+            bb = doc.getBoundingBox(true, true);
+        }
         properties["width"] = Math.ceil(bb.getWidth() * properties["resolution"] + 2 * properties["margin"]);
         properties["height"] = Math.ceil(bb.getHeight() * properties["resolution"] + 2 * properties["margin"]);
     }
@@ -98,18 +112,34 @@ function exportBitmap(doc, scene, fileName, properties, view) {
     if (properties["window"]) {
         view.zoomTo(properties["window"], properties["margin"]);
     }
+    else if (properties["zoomall"]) {
+        var bbz = doc.getBoundingBox(false);
+        view.zoomTo(bbz, properties["margin"]);
+    }
+    else if (typeof(properties["entityids"])!=="undefined") {
+        view.zoomToEntities(properties["entityids"], properties["margin"]);
+    }
     else {
         view.autoZoom(properties["margin"], true, properties["noweightmargin"]);
+    }
 
-        // make sure we use the desired resolution:
-        // auto zoom might be slightly off, due to rounding canvas to pixels:
-        if (properties["resolution"]) {
-            view.setFactor(properties["resolution"]);
-        }
+    // make sure we use the desired resolution:
+    // auto zoom might be slightly off, due to rounding canvas to pixels:
+    if (properties["resolution"]) {
+        view.setFactor(properties["resolution"]);
     }
 
     view.clear();
-    scene.regenerate();
+    view.setPrinting(true);
+
+    if (properties["regen"]!==false) {
+        scene.regenerate();
+        view.repaintView();
+    }
+
+    if (isFunction(properties["initView"])) {
+        properties["initView"](view);
+    }
 
     // export file
     var buffer = view.getBuffer();
