@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -52,8 +52,11 @@ function PrintPreview(guiAction) {
 PrintPreview.prototype = new DefaultAction();
 PrintPreview.includeBasePath = includeBasePath;
 
-if (typeof(printPreviewRunning)==="undefined") {
-    printPreviewRunning = false;
+if (typeof(global.printPreviewRunning)==="undefined") {
+    global.printPreviewRunning = false;
+}
+if (typeof(global.printPreviewInstance)==="undefined") {
+    global.printPreviewInstance = undefined;
 }
 
 /**
@@ -64,19 +67,33 @@ PrintPreview.State = {
 };
 
 PrintPreview.isRunning = function() {
-    return printPreviewRunning;
+    return global.printPreviewRunning;
 };
 
 PrintPreview.getInstance = function() {
-    return printPreviewInstance;
+    return global.printPreviewInstance;
+};
+
+PrintPreview.setRunning = function(v) {
+    global.printPreviewRunning = v;
+};
+
+PrintPreview.setInstance = function(inst) {
+    global.printPreviewInstance = inst;
+};
+
+PrintPreview.terminate = function() {
+    var inst = PrintPreview.getInstance();
+    if (!isNull(inst)) {
+        inst.terminate();
+    }
 };
 
 PrintPreview.prototype.beginEvent = function() {
+    var di = this.getDocumentInterface();
     if (PrintPreview.isRunning()) {
         // allows use of print preview button to close print preview:
-        if (!isNull(printPreviewInstance)) {
-            printPreviewInstance.terminate();
-        }
+        PrintPreview.terminate();
         if (!isNull(this.guiAction)) {
             this.guiAction.setChecked(false);
         }
@@ -94,8 +111,8 @@ PrintPreview.prototype.beginEvent = function() {
     }
 
     // globals:
-    printPreviewRunning = true;
-    printPreviewInstance = this;
+    PrintPreview.setRunning(true);
+    PrintPreview.setInstance(this);
 
     var mdiChild = EAction.getMdiChild();
     this.view = mdiChild.getLastKnownViewWithFocus();
@@ -106,13 +123,6 @@ PrintPreview.prototype.beginEvent = function() {
         var document = this.getDocument();
         this.bgColor = this.view.getBackgroundColor();
 
-        // background needs to be set to white (or other user preference)
-        // for printing / preview:
-        this.view.setBackgroundColor(Print.getBackgroundColor(document));
-        this.view.setColorMode(Print.getColorMode(document));
-        this.view.setHairlineMode(Print.getHairlineMode(document));
-        this.view.setPrintPreview(true);
-
         this.updateBackgroundDecoration();
 
         if (!isNull(this.view.getScene())) {
@@ -125,7 +135,6 @@ PrintPreview.prototype.beginEvent = function() {
         }
 
         // needed to update pattern scaling according to drawing scale:
-        var di = EAction.getDocumentInterface();
         di.regenerateScenes();
     }
 
@@ -147,6 +156,18 @@ PrintPreview.prototype.beginEvent = function() {
     else if (this.initialAction==="PdfExport") {
         this.slotPdfExport();
     }
+
+    // create listener to update preview when preferences changed:
+    this.pAdapter = new RPreferencesListenerAdapter();
+    appWin.addPreferencesListener(this.pAdapter);
+    this.pAdapter.preferencesUpdated.connect(this, "updateFromPreferences");
+
+    this.bAdapter = new RBlockListenerAdapter();
+    appWin.addBlockListener(this.bAdapter);
+    this.bAdapter.blocksUpdated.connect(this, "updateBackgroundTransform");
+    this.bAdapter.currentBlockSet.connect(this, "updateBackgroundTransform");
+    this.bAdapter.blocksUpdated.connect(this, "updateBackgroundDecoration");
+    this.bAdapter.currentBlockSet.connect(this, "updateBackgroundDecoration");
 };
 
 /**
@@ -176,7 +197,8 @@ PrintPreview.prototype.suspendEvent = function() {
 
 PrintPreview.prototype.finishEvent = function() {
     DefaultAction.prototype.finishEvent.call(this);
-        
+    var di = this.getDocumentInterface();
+
     if (!isNull(this.view)) {
         this.view.setPrintPreview(false);
         this.view.setBackgroundColor(this.bgColor);
@@ -192,24 +214,33 @@ PrintPreview.prototype.finishEvent = function() {
         }
 
         // needed to update pattern scaling without using drawing scale:
-        var di = EAction.getDocumentInterface();
-        di.regenerateScenes();
+        if (!isNull(di)) {
+            di.regenerateScenes();
+        }
 
         var action = RGuiAction.getByScriptFile("scripts/Edit/DrawingPreferences/DrawingPreferences.js");
         action.triggered.disconnect(this, "updateBackgroundDecoration");
     }
 
-    printPreviewRunning = false;
-    printPreviewInstance = undefined;
+    PrintPreview.setRunning(false);
+    PrintPreview.setInstance(undefined);
 
     if (this.saveView===true) {
         if (!isNull(this.savedScale)) {
-            Print.setScale(this.getDocument(), this.savedScale);
+            Print.setScale(di.getDocument(), this.savedScale);
         }
         if (!isNull(this.savedOffset)) {
-            Print.setOffset(this.getDocument(), this.savedOffset);
+            Print.setOffset(di.getDocument(), this.savedOffset);
         }
     }
+
+    var appWin = RMainWindowQt.getMainWindow();
+    if (!isNull(this.pAdapter)) {
+        appWin.removePreferencesListener(this.pAdapter);
+    }
+    if (!isNull(this.bAdapter)) {
+        appWin.removeBlockListener(this.bAdapter);
+    }
 };
 
 /**
@@ -344,7 +375,7 @@ PrintPreview.prototype.showUiOptions = function(resume) {
 
     widgets["Portrait"].blockSignals(true);
     widgets["Landscape"].blockSignals(true);
-    if (Print.getPageOrientationEnum() == QPrinter.Portrait) {
+    if (Print.getPageOrientationEnum(document).valueOf() === QPrinter.Portrait.valueOf()) {
         widgets["Portrait"].checked=true;
     } else {
         widgets["Landscape"].checked=true;
@@ -370,7 +401,7 @@ PrintPreview.prototype.initScaleCombo = function() {
     for (var i=0; i<scales.length; ++i) {
         scaleCombo.addItem(scales[i]);
     }
-    scaleCombo.currentText = prev;
+    scaleCombo.setEditText(prev);
     scaleCombo.blockSignals(false);
 };
 
@@ -383,9 +414,20 @@ PrintPreview.prototype.hideUiOptions = function() {
  * or paper settings change.
  */
 PrintPreview.prototype.updateBackgroundDecoration = function() {
+    var document = this.getDocument();
+
+    if (isNull(document) || isNull(this.view)) {
+        return;
+    }
+
+    // initialize graphics view for printing / print preview:
+    this.view.setBackgroundColor(Print.getBackgroundColor(document));
+    this.view.setColorMode(Print.getColorMode(document));
+    this.view.setHairlineMode(Print.getHairlineMode(document));
+    this.view.setPrintPreview(true);
+
     this.view.clearBackground();
 
-    var document = this.getDocument();
     var pages = Print.getPages(document);
     var backgroundColor = Print.getBackgroundColor(document);
     var i, color, path;
@@ -478,6 +520,11 @@ PrintPreview.prototype.addDecorations = function(pages) {};
  */
 PrintPreview.prototype.updateBackgroundTransform = function() {
     var document = EAction.getDocument();
+
+    if (isNull(document)) {
+        return;
+    }
+
     var scale = Print.getScale(document);
     var offset = Print.getOffset(document);
 
@@ -486,7 +533,9 @@ PrintPreview.prototype.updateBackgroundTransform = function() {
     var unitScale = RUnit.convert(1.0, RS.Millimeter, document.getUnit());
     var factor = 1.0 / scale * unitScale;
 
-    this.view.setBackgroundTransform(factor, offset);
+    if (!isNull(this.view)) {
+        this.view.setBackgroundTransform(factor, offset);
+    }
 };
 
 /**
@@ -526,13 +575,6 @@ PrintPreview.prototype.drawShadow = function(path, border) {
 };
 
 PrintPreview.prototype.slotPdfExport = function() {
-    PrintPreview.slotPdfExport();
-};
-
-/**
- * Called when user clicks the PDF export button in the options toolbar.
- */
-PrintPreview.slotPdfExport = function() {
     var appWin = EAction.getMainWindow();
     var fileName = EAction.getDocument().getFileName();
     var initialFileName = QDir.homePath();
@@ -554,7 +596,7 @@ PrintPreview.slotPdfExport = function() {
 
     appWin.handleUserMessage(qsTr("Exporting to %1...").arg(pdfFile));
 
-    var success = PrintPreview.slotPrint(pdfFile);
+    var success = this.slotPrint(pdfFile);
 
     if (success) {
         appWin.handleUserMessage(qsTr("Export complete: %1").arg(pdfFile));
@@ -564,6 +606,14 @@ PrintPreview.slotPdfExport = function() {
     }
 };
 
+/**
+ * Called when user clicks the PDF export button in the options toolbar.
+ */
+PrintPreview.slotPdfExport = function() {
+    var pp = PrintPreview.getInstance();
+    pp.slotPdfExport();
+};
+
 PrintPreview.prototype.slotPrint = function(pdfFile) {
     return PrintPreview.slotPrint(pdfFile);
 };
@@ -616,7 +666,7 @@ PrintPreview.updateScaleString = function(document) {
     var scaleCombo = optionsToolBar.findChild("Scale");
     if (Print.getScale(document) != RMath.parseScale(scaleCombo.currentText)) {
         scaleCombo.blockSignals(true);
-        scaleCombo.setEditText(Print.getScaleString());
+        scaleCombo.setEditText(Print.getScaleString(document));
         scaleCombo.blockSignals(false);
     }
 };
@@ -682,7 +732,7 @@ PrintPreview.prototype.slotGrayscaleChanged = function(checked) {
 
 PrintPreview.prototype.colorModeUpdate = function() {
     if (!isNull(this.view)) {
-        this.view.setColorMode(Print.getColorMode());
+        this.view.setColorMode(Print.getColorMode(EAction.getDocument()));
         this.view.regenerate(true);
     }    
 };
@@ -693,7 +743,6 @@ PrintPreview.prototype.slotHairlineChanged = function(checked) {
     }
     Print.setHairlineMode(EAction.getDocument(), checked);
     if (!isNull(this.view)) {
-        qDebug("hairline mode: ", checked);
         this.view.setHairlineMode(checked)
         this.view.regenerate(true);
     }
@@ -741,6 +790,7 @@ PrintPreview.prototype.slotAutoFitBox = function(box) {
     this.updateBackgroundTransform();
     //this.slotAutoZoomToPage();
     this.updateBackgroundDecoration();
+    this.showUiOptions(true);
 };
 
 /**
@@ -758,6 +808,7 @@ PrintPreview.prototype.slotAutoFitDrawing = function() {
     this.updateBackgroundTransform();
     this.slotAutoZoomToPage();
     this.updateBackgroundDecoration();
+    this.showUiOptions(true);
 };
 
 PrintPreview.prototype.slotAutoCenter = function() {
@@ -804,3 +855,12 @@ PrintPreview.prototype.slotAutoZoomToPage = function() {
     }
 };
 
+/**
+ * Updates the background (paper preview) transform when page settings
+ * are changed (e.g. offset).
+ */
+PrintPreview.prototype.updateFromPreferences = function() {
+    this.showUiOptions(true);
+    this.updateBackgroundDecoration();
+    this.updateBackgroundTransform();
+};
