@@ -66,6 +66,7 @@ function RBlockListQt(parent, addListener, showHeader) {
         var adapter = new RBlockListenerAdapter();
         appWin.addBlockListener(adapter);
         adapter.blocksUpdated.connect(this, "updateBlocks");
+        adapter.currentBlockSet.connect(this, "updateBlocks");
         adapter.blocksCleared.connect(this, "clearBlocks");
     }
 
@@ -77,7 +78,7 @@ function RBlockListQt(parent, addListener, showHeader) {
 
 RBlockListQt.prototype = new RTreeWidget();
 
-RBlockListQt.getBlockList = function() {
+RBlockListQt.getWidget = function() {
     var appWin = EAction.getMainWindow();
     return appWin.findChild("BlockList");
 };
@@ -86,19 +87,19 @@ RBlockListQt.getBlockList = function() {
  * Add an additional action to be used in the context menu.
  * Can be used by plugins to hook into the block list context menu.
  */
-RBlockListQt.addContextMenuScriptFile = function(scriptFile) {
-    var blockList = RBlockListQt.getBlockList();
+RBlockListQt.addContextMenuScriptFile = function(scriptFile, blockList) {
+    //var blockList = RBlockListQt.getWidget();
     if (isNull(blockList)) {
         return;
     }
 
-    var scriptFiles = RBlockListQt.getContextMenuScriptFiles();
+    var scriptFiles = RBlockListQt.getContextMenuScriptFiles(blockList);
     scriptFiles.push(scriptFile);
     blockList.setProperty("ContextMenuScriptFiles", scriptFiles);
 };
 
-RBlockListQt.getContextMenuScriptFiles = function() {
-    var blockList = RBlockListQt.getBlockList();
+RBlockListQt.getContextMenuScriptFiles = function(blockList) {
+    //var blockList = RBlockListQt.getWidget();
     if (isNull(blockList)) {
         return [];
     }
@@ -129,8 +130,16 @@ RBlockListQt.prototype.contextMenuEvent = function(e) {
     RGuiAction.getByScriptFile("scripts/Block/SelectBlockReferences/SelectBlockReferences.js").addToMenu(menu);
     RGuiAction.getByScriptFile("scripts/Block/DeselectBlockReferences/DeselectBlockReferences.js").addToMenu(menu);
 
+    RBlockListQt.complementContextMenu(menu, RBlockListQt.getWidget());
+
+    menu.exec(QCursor.pos());
+
+    e.ignore();
+};
+
+RBlockListQt.complementContextMenu = function(menu, blockListWidget) {
     // add addtional context menu actions provided by plugins:
-    var scriptFiles = RBlockListQt.getContextMenuScriptFiles();
+    var scriptFiles = RBlockListQt.getContextMenuScriptFiles(blockListWidget);
     for (var i=0; i<scriptFiles.length; i++) {
         var scriptFile = scriptFiles[i];
         var a = RGuiAction.getByScriptFile(scriptFile);
@@ -138,10 +147,6 @@ RBlockListQt.prototype.contextMenuEvent = function(e) {
             a.addToMenu(menu);
         }
     }
-
-    menu.exec(QCursor.pos());
-
-    e.ignore();
 };
 
 RBlockListQt.prototype.filter = function(block) {
@@ -166,60 +171,6 @@ RBlockListQt.prototype.filter = function(block) {
     return true;
 };
 
-RBlockListQt.prototype.sortBlockNames = function(blockNames) {
-    var doc = this.di.getDocument();
-
-    //var self = this;
-    blockNames.sort(function(a, b) {
-        var blockA = doc.queryBlock(a);
-        var blockB = doc.queryBlock(b);
-        if (isNull(blockA) || isNull(blockB)) {
-            return 0;
-        }
-
-        // model space always first:
-        if (blockA.getName()===RBlock.modelSpaceName) {
-            return -1;
-        }
-        if (blockB.getName()===RBlock.modelSpaceName) {
-            return 1;
-        }
-
-        // layouts always before other blocks:
-        var blockAIsLayout = blockA.isLayout();
-        var blockBIsLayout = blockB.isLayout();
-        if (blockAIsLayout && !blockBIsLayout) {
-            return -1;
-        }
-        if (!blockAIsLayout && blockBIsLayout) {
-            return 1;
-        }
-
-        // sort among layouts by tab order:
-        if (blockAIsLayout && blockBIsLayout) {
-            var layoutAId = blockA.getLayoutId();
-            var layoutBId = blockB.getLayoutId();
-            var layoutA = doc.queryLayout(layoutAId);
-            var layoutB = doc.queryLayout(layoutBId);
-            if (!isNull(layoutA) && !isNull(layoutB)) {
-                return layoutA.getTabOrder() - layoutB.getTabOrder();
-            }
-        }
-
-        // sort by layout name or block name (ignoring * at start):
-        var blockTitleA = RBlockListQt.getBlockTitle(blockA);
-        var blockTitleB = RBlockListQt.getBlockTitle(blockB);
-        if (blockTitleA.startsWith("*")) {
-            blockTitleA = blockTitleA.substring(1);
-        }
-        if (blockTitleB.startsWith("*")) {
-            blockTitleB = blockTitleB.substring(1);
-        }
-
-        return Array.alphaNumericalSorter(blockTitleA, blockTitleB);
-    });
-};
-
 /**
  * Called when blocks are addded, edited or deleted. Updates the block list.
  */
@@ -240,15 +191,15 @@ RBlockListQt.prototype.updateBlocks = function(documentInterface) {
 
     var doc = documentInterface.getDocument();
 
-    var blockNameCandidates = doc.getBlockNames();
+    var blockCandidateIds = doc.queryAllBlocks();
 
     var i;
 
     // filter block names:
-    var blockNames = [];
-    for (i=0; i<blockNameCandidates.length; i++) {
-        var blockNameCandidate = blockNameCandidates[i];
-        var blockCandidate = doc.queryBlockDirect(blockNameCandidate);
+    var blockIds = [];
+    for (i=0; i<blockCandidateIds.length; i++) {
+        var blockCandidateId = blockCandidateIds[i];
+        var blockCandidate = doc.queryBlockDirect(blockCandidateId);
         if (blockCandidate.isNull()) {
             continue;
         }
@@ -257,22 +208,24 @@ RBlockListQt.prototype.updateBlocks = function(documentInterface) {
             continue;
         }
 
-        blockNames.push(blockNameCandidate);
+        blockIds.push(blockCandidateId);
     }
 
-    this.sortBlockNames(blockNames);
+    blockIds = this.sortBlocks(doc, blockIds);
 
     var currentBlockId = doc.getCurrentBlockId();
     var currentItem = undefined;
     var selectedItem = undefined;
-    for (i=0; i<blockNames.length; ++i) {
-        var blockName = blockNames[i];
-        var block = doc.queryBlockDirect(blockName);
+    for (i=0; i<blockIds.length; ++i) {
+        var blockId = blockIds[i];
+        var block = doc.queryBlockDirect(blockId);
         if (block.isNull()) {
             continue;
         }
 
-        var item = this.getBlockItem(block);
+        var blockName = block.getName();
+
+        var item = this.createBlockItem(block);
         this.addTopLevelItem(item);
         if (blockName===selectedBlockName) {
             selectedItem = item;
@@ -294,6 +247,10 @@ RBlockListQt.prototype.updateBlocks = function(documentInterface) {
     this.blockActivated();
 };
 
+RBlockListQt.prototype.sortBlocks = function(doc, blockIds) {
+    return doc.sortBlocks(blockIds);
+};
+
 RBlockListQt.getBlockTitle = function(block) {
     var title = block.getName();
     var layoutName = block.getLayoutName();
@@ -303,7 +260,7 @@ RBlockListQt.getBlockTitle = function(block) {
     return title;
 };
 
-RBlockListQt.prototype.getBlockItem = function(block) {
+RBlockListQt.prototype.createBlockItem = function(block) {
     var item = new QTreeWidgetItem();
     var name = block.getName();
 
@@ -404,7 +361,9 @@ RBlockListQt.prototype.blockActivated = function() {
 
     // prevent removal or insertion of current block:
     var insertable = currentBlock.getName().toLowerCase()!==blockName.toLowerCase() && !blockName.startsWith("*");
-    this.enableActions(insertable);
+    var renamable = document.getBlockId(blockName)!==document.getModelSpaceBlockId();
+    var removable = document.getBlockId(blockName)!==document.getModelSpaceBlockId();
+    this.enableActions(insertable, renamable, removable);
 };
 
 /**
@@ -414,12 +373,32 @@ RBlockListQt.prototype.getBlockName = function(item) {
     return item.data(BlockList.colName, Qt.UserRole);
 };
 
+/**
+ * \return The list item for the block with the given block name.
+ */
+RBlockListQt.getItem = function(widget, blockName) {
+    for (var i=0; i<widget.topLevelItemCount; i++) {
+        var item = widget.topLevelItem(i);
+        if (item.data(BlockList.colName, Qt.UserRole)===blockName) {
+            return item;
+        }
+    }
+    return undefined;
+};
+
 /**
  * Enable / disable buttons that edit, remove or insert the active block.
  */
-RBlockListQt.prototype.enableActions = function(enable) {
+RBlockListQt.prototype.enableActions = function(insertable, renamable, removable) {
+    if (isNull(renamable)) {
+        renamable = insertable;
+    }
+    if (isNull(removable)) {
+        removable = insertable;
+    }
+
     var override;
-    if (enable===true) {
+    if (insertable===true) {
         override = 1;
     }
     else {
@@ -428,13 +407,15 @@ RBlockListQt.prototype.enableActions = function(enable) {
 
     var action;
     action = RGuiAction.getByScriptFile("scripts/Block/ToggleBlockVisibility/ToggleBlockVisibility.js");
-    action.setEnabledOverride(enable, override);
+    action.setEnabledOverride(insertable, override);
     action = RGuiAction.getByScriptFile("scripts/Block/InsertBlock/InsertBlock.js");
-    action.setEnabledOverride(enable, override);
+    action.setEnabledOverride(insertable, override);
+
     action = RGuiAction.getByScriptFile("scripts/Block/RemoveBlock/RemoveBlock.js");
-    action.setEnabledOverride(enable, override);
+    action.setEnabledOverride(removable, removable ? 1 : 0);
+
     action = RGuiAction.getByScriptFile("scripts/Block/RenameBlock/RenameBlock.js");
-    action.setEnabledOverride(enable, override);
+    action.setEnabledOverride(renamable, renamable ? 1 : 0);
 };
 
 /**
